#! /bin/bash

# MSP430 firmware version
REQ_VERSION=2.1

find_mtdnum() {
    # mtdname -> mtd number
    local mtdname=$1
    local mtd=`cat /proc/mtd | grep -F \"$mtdname\" | cut -d ':' -f 1`
    local mtdnumber="${mtd#mtd}"

    echo $mtdnumber
}

write_to_mtd() {
    # mountpoint, kind, area, pre-erase
    local dir=$1
    local kind=$2
    local area=$3
    local erase=$4

    local mtdname=

    if [ "$area" = "0" ] || [ "$area" = "1" ]; then
        mtdname=$kind.$area
    elif [ "$area" == "-1" ]; then
        mtdname=$kind
    else
        mtdname=$area
    fi

    local mtdnum=`find_mtdnum $mtdname`

    local mtdblk="/dev/mtdblock$mtdnum"
    local mmcblk=

    if [ "$mtdnum" = "" ]; then
        echo "  ! MTD \"$mtdname\" not found."
        cleanup $dir
        exit 1
    fi

    if [ "$erase" = "1" ]; then
        # MTD
        echo -n "  writing \"$kind\" image to /dev/mtd$mtdnum ... "
        flashcp $dir/$kind.img /dev/mtd$mtdnum &> /dev/null
        res=$?
    fi

    if [ "$res" != "0" ]; then
        echo "failed."
        exit 1
    else
        echo "succeeded."
    fi
}

set_led() {
    # led dir, value
    local dir=$1
    local val=$2

    echo $val > $dir/brightness
}

MNTPOINT=$1
AREA=$2

if [ -f ${MNTPOINT}/at91bootstrap.bin.img ]; then
    echo "* AT91Bootstrap file found."
    write_to_mtd ${MNTPOINT} at91bootstrap.bin at91boot 1
fi

# find DIO MSP430's RESET LEDs
i2cget -y 1 0x48 0 &> /dev/null
if [ "$?" = "0" ]; then
    LEDS=`find /sys/class/leds -name "MSP430_*_RST"`
fi
if [ "${LEDS}" != "" ]; then
    NEED_MSP430_RESET=1
else
    NEED_MSP430_RESET=0
fi

${MNTPOINT}/msp430_ctrl check_version ${REQ_VERSION} &> /dev/null
if [ "$?" != "0" ]; then
    echo "* MSP430 needs to be programmed."

    if [ "${NEED_MSP430_RESET}" = "1" ]; then
        for led in "${LEDS}"; do
            echo "  Assert Reset: ${led##*/}"
            set_led $led 255
        done;
    fi

    ${MNTPOINT}/msp430_writer -f ${MNTPOINT}/msp430firm.txt

    if [ "${NEED_MSP430_RESET}" = "1" ]; then
        for led in "${LEDS}"; do
            echo "  De-Assert Reset: ${led##*/}"
            set_led $led 0
        done;
    fi
else
    echo "MSP430 is not need to update firmware."
fi

MSP430_XIO_FIRM=${MNTPOINT}/msp430dio.txt
if [ -f ${MSP430_XIO_FIRM} ]; then
    for idx in `seq 1 1`; do
        LED="/sys/class/leds/MSP430_EXT${idx}_RST"
        if [ ! -e ${LED} ]; then
            continue
        fi
        echo "* Update XIO MSP430(${idx}) firmware."
        ${MNTPOINT}/msp430_writer -f ${MSP430_XIO_FIRM} -b 1 -s EXT${idx}
    done
fi
